# I will try and make this quality checks file as standard as possible
# so that it can be used in a lot of different scenarios
# It will take in the directory it needs to check and then perform multiple different 
# checks on the data.

import sys
import os
import argparse
import json
import pandas as pd
import xarray as xr
import numpy as np
import netCDF4 as nc4
from netCDF4 import Dataset as nc
from io import StringIO
from quality_checks import Checks
from datetime import datetime, timedelta
import paramiko

standard_parameters = {

    "O3": {'lower_limit':1e-13, 'upper_limit':7.9828e-07,},
    "NO2": {'lower_limit':1e-13, 'upper_limit':1.1475e-06,},
    "CO": {'lower_limit':1e-11, 'upper_limit':3.4926e-05,},
    "SO2": {'lower_limit':1e-12, 'upper_limit':7.982699999999999e-06,},
    "PM10": {'lower_limit':3e-13, 'upper_limit':3e-06,},
    "PM25": {'lower_limit':2e-13, 'upper_limit':2e-06,},

}


class EionetMeteofranceChecks(Checks):
    """
    Checks specific to Eionet, this could possibly go in a separate file 
    """
    def __init__(self, indir, file, date):
        super().__init__(indir, file)
        self.dataset = "Eionet Meteofrance"
        self.date = date


    def check_micrograms(self):
        df = pd.read_csv(self.filepath, engine='python', header=0, encoding= 'unicode_escape')
        odd_values = False
        odd_values_dict = {}

        # function to add to JSON
        def write_json(date, new_data, filename='erroneous-station-values.json'):
            with open(filename,'r+') as file:
                # First we load existing data into a dict.
                file_data = json.load(file)
                # Join new_data with file_data inside emp_details
                if date in file_data.keys():
                    print("Date already exists in file")
                else:
                    print("Adding date to erroneous dates file")
                    file_data[date] = new_data

                # Sets file's current position at offset.
                file.seek(0)
                # convert back to json.
                json.dump(file_data, file, indent = 4)

        for element, values in standard_parameters.items():
            element_df = df[df["PARAMETER"] == element]
            cut_df = element_df[~element_df["CONCENTRATION"].between(values["lower_limit"], values["upper_limit"])]
            odd_values_dict[element] = list(set(cut_df["STATION"].to_list()))
            if not cut_df.empty:
                odd_values = True
                write_json(self.date, odd_values_dict)


        if odd_values:
            return "Some stations in the csv have erroneous values which may indicate wrong units {}".format(odd_values_dict)
        

    def eionet_original_files_check(self):
        error_list = []
        file_exists = self.check_that_file_exists()
        print(self.filepath)
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:

            file_size = self.check_file_size()
            if file_size != None:
                error_list.append(file_size)
    
            stations = self.check_micrograms()
            if stations != None:
                error_list.append(stations)
    
            if error_list != []:
                self.send_email_over_ssh(error_list)
            else:
                print("No errors reported")


    def eionet_formatted_files_check(self):
        """
        Checking formatted Eionet meteofrance files at 
        """
        error_list = []
        # First check that the file exists
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:
            # If the file exist next check that it hasn't been corrupted
            file_corrupted = self.check_corrupted_netcdf()
            if file_corrupted != None:
                error_list.append(file_corrupted)
                self.send_email_over_ssh(error_list)
            else:
                #If not corrupted continue with further checks
                file_size = self.check_file_size()
                if file_size != None:
                    error_list.append(file_size)
    
                if error_list != []:
                    print(error_list)
                else:
                    print("No errors reported")



if __name__ == "__main__":
    print(sys.argv)

    # Original files check
    #options = getOptions(sys.argv[1:])
    today = datetime.today()
    yesterday = (today - timedelta(days=1)).strftime("%Y%m%d")
    indir = "/esarchive/obs/meteofrance/eionet-cams2_40/original_files"#options.indir
    file = "obsmacc4ana_{}.csv".format(yesterday)
    eio_ori = EionetMeteofranceChecks(indir, file, yesterday)
    eio_ori.eionet_original_files_check()

    # Formatted file check
    date = today.strftime("%Y%m")
    path = "/esarchive/obs/meteofrance/eionet-cams2_40/hourly"
    elements = ["pm10", "pm2p5", "sconcco", "sconcno2", "sconco3", "sconcso2"]
    for element in elements:
        indir = path + "/{}".format(element)
        file = "{}_{}.nc".format(element, date)
        print("Checking file {}".format(file))
        eio_fmt = EionetMeteofranceChecks(indir, file)
        eio_fmt.eionet_formatted_files_check()
    
    sys.exit()

